package vieon.reactlibrary.ima;

import android.util.Log;
import android.widget.FrameLayout;
import com.facebook.react.bridge.LifecycleEventListener;
import com.facebook.react.uimanager.ThemedReactContext;
import com.google.ads.interactivemedia.v3.api.AdDisplayContainer;
import com.google.ads.interactivemedia.v3.api.AdErrorEvent;
import com.google.ads.interactivemedia.v3.api.AdEvent;
import com.google.ads.interactivemedia.v3.api.AdsLoader;
import com.google.ads.interactivemedia.v3.api.AdsLoader.AdsLoadedListener;
import com.google.ads.interactivemedia.v3.api.AdsManager;
import com.google.ads.interactivemedia.v3.api.AdsManagerLoadedEvent;
import com.google.ads.interactivemedia.v3.api.AdsRequest;
import com.google.ads.interactivemedia.v3.api.ImaSdkFactory;
import com.google.ads.interactivemedia.v3.api.player.ContentProgressProvider;
import com.google.ads.interactivemedia.v3.api.player.VideoProgressUpdate;

public class ReactIMAWrapperView extends FrameLayout
    implements LifecycleEventListener, AdEvent.AdEventListener, AdErrorEvent.AdErrorListener,
    AdsLoadedListener {

  private final ThemedReactContext themedReactContext;
  private final ReactEventEmitter eventEmitter;

  private ImaSdkFactory mSdkFactory;
  private AdsLoader mAdsLoader;
  private AdsManager mAdsManager;
  private boolean mIsAdDisplayed;

  public ReactIMAWrapperView(ThemedReactContext context) {
    super(context);
    this.themedReactContext = context;
    this.eventEmitter = new ReactEventEmitter(context);
    themedReactContext.addLifecycleEventListener(this);
  }

  public void startAds() {
    if (mAdsManager != null) mAdsManager.start();
  }

  @Override
  public void setId(int id) {
    super.setId(id);
    eventEmitter.setViewId(id);
  }

  @Override
  protected void onAttachedToWindow() {
    super.onAttachedToWindow();

  }

  @Override
  protected void onDetachedFromWindow() {
    super.onDetachedFromWindow();
    removeAllListeners();
  }

  @Override
  public void requestLayout() {
    super.requestLayout();
    post(measureAndLayout);
  }

  private final Runnable measureAndLayout = new Runnable() {
    @Override
    public void run() {
      measure(
          MeasureSpec.makeMeasureSpec(getWidth(), MeasureSpec.EXACTLY),
          MeasureSpec.makeMeasureSpec(getHeight(), MeasureSpec.EXACTLY));
      layout(getLeft(), getTop(), getRight(), getBottom());
    }
  };

  public void requestAds(String adTagUrl, final int currentPosition, final int duration) {
    if (adTagUrl != null) {
      initializeAds();
      Log.i("IMA", "Ad Tag URL: " + adTagUrl);
      AdDisplayContainer adDisplayContainer = mSdkFactory.createAdDisplayContainer();
      adDisplayContainer.setAdContainer(this);

      // Create the ads request.
      AdsRequest request = mSdkFactory.createAdsRequest();
      request.setAdTagUrl(adTagUrl);
      request.setVastLoadTimeout(15);
      request.setAdDisplayContainer(adDisplayContainer);
      request.setContentProgressProvider(new ContentProgressProvider() {
        @Override
        public VideoProgressUpdate getContentProgress() {
          if (mIsAdDisplayed && currentPosition >=0 && duration >0) {
            return VideoProgressUpdate.VIDEO_TIME_NOT_READY;
          }
          return new VideoProgressUpdate(currentPosition,duration);
        }
      });

      // Request the ad. After the ad is loaded, onAdsManagerLoaded() will be called.
      mAdsLoader.requestAds(request);
    }
  }

  public void removeAllListeners() {
    if (mAdsLoader != null) {
      mAdsLoader.removeAdErrorListener(ReactIMAWrapperView.this);
      mAdsLoader.removeAdsLoadedListener(ReactIMAWrapperView.this);
    }

    if (mAdsManager != null) {
      mAdsManager.removeAdErrorListener(ReactIMAWrapperView.this);
      mAdsManager.removeAdEventListener(ReactIMAWrapperView.this);
    }
    if(themedReactContext!=null) themedReactContext.removeLifecycleEventListener(this);
  }

  private void initializeAds() {
    // Create an AdsLoader.
    mSdkFactory = ImaSdkFactory.getInstance();
    mAdsLoader = mSdkFactory.createAdsLoader(this.getContext());
    // Add listeners for when ads are loaded and for errors.
    mAdsLoader.addAdErrorListener(this);
    mAdsLoader.addAdsLoadedListener(this);
  }

  @Override public void onHostResume() {
    if (mAdsManager != null) {
      Log.i("IMA", "mAdsManager: resume");
      mAdsManager.resume();
    }
  }

  @Override public void onHostPause() {
    if (mAdsManager != null) {
      Log.i("IMA", "mAdsManager: pause");
      mAdsManager.pause();
    }
  }

  @Override public void onHostDestroy() {
    removeAllListeners();
  }

  @Override public void onAdError(AdErrorEvent adErrorEvent) {
    Log.e("IMA", "Ad Error: " + adErrorEvent.getError().getMessage());
    eventEmitter.adError();
  }

  @Override public void onAdEvent(AdEvent adEvent) {
    Log.i("IMA", "Event: " + adEvent.getType());

    // These are the suggested event types to handle. For full list of all ad event
    // types, see the documentation for AdEvent.AdEventType.
    switch (adEvent.getType()) {
      case LOADED:
        // AdEventType.LOADED will be fired when ads are ready to be played.
        // AdsManager.start() begins ad playback. This method is ignored for VMAP or
        // ad rules playlists, as the SDK will automatically start executing the
        // playlist.
        eventEmitter.adsLoaded();
        break;
      case CONTENT_PAUSE_REQUESTED:
        // AdEventType.CONTENT_PAUSE_REQUESTED is fired immediately before a video
        // ad is played.
        mIsAdDisplayed = true;
        break;
      case STARTED:
        eventEmitter.adStarted();
        break;
      case CONTENT_RESUME_REQUESTED:
        // AdEventType.CONTENT_RESUME_REQUESTED is fired when the ad is completed
        // and you should start playing your content.
        mIsAdDisplayed = false;
        break;
      case ALL_ADS_COMPLETED:
        if (mAdsManager != null) {
          mAdsManager.destroy();
          mAdsManager = null;
        }
        eventEmitter.adsCompleted();
        break;
      default:
        break;
    }
  }

  @Override public void onAdsManagerLoaded(AdsManagerLoadedEvent adsManagerLoadedEvent) {
    mAdsManager = adsManagerLoadedEvent.getAdsManager();

    // Attach event and error event listeners.
    mAdsManager.addAdErrorListener(ReactIMAWrapperView.this);
    mAdsManager.addAdEventListener(ReactIMAWrapperView.this);
    mAdsManager.init();
  }
}
